@@ -108,24 +108,45 @@ void applet_size_allocate_cb(GtkWidget *widget, GtkAllocation *allocation, Stick
 }
 
 /* Applet Callback : Change the applet background. */
-gboolean applet_change_bg_cb(PanelApplet *panel_applet, PanelAppletBackgroundType type, GdkColor *color, GdkPixmap *pixmap,
-			     StickyNotesApplet *applet)
+void
+applet_change_bg_cb (PanelApplet *panel_applet,
+		     PanelAppletBackgroundType type,
+		     GdkColor *color,
+		     GdkPixmap *pixmap,
+		     StickyNotesApplet *applet)
 {
-	if (type == PANEL_NO_BACKGROUND) {
-		GtkRcStyle *rc_style = gtk_rc_style_new();
-		gtk_widget_modify_style(applet->w_applet, rc_style);
-		gtk_rc_style_unref(rc_style);
-	}
+	/* Taken from TrashApplet */
+	GtkRcStyle *rc_style;
+	GtkStyle *style;
 
-	else if (type == PANEL_COLOR_BACKGROUND) {
-		gtk_widget_modify_bg(applet->w_applet, GTK_STATE_NORMAL, color);
-	}
+	if (!applet) g_print ("arrg, no applet!\n");
 
-	else /* if (type == PANEL_PIXMAP_BACKGROUND) */ {
-		gdk_window_set_back_pixmap(applet->w_applet->window, pixmap, FALSE); /* FIXME : Does not work */
+	/* reset style */
+	gtk_widget_set_style (GTK_WIDGET (applet->w_applet), NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (applet->w_applet), rc_style);
+	g_object_unref (rc_style);
+
+	switch (type)
+	{
+		case PANEL_NO_BACKGROUND:
+			break;
+		case PANEL_COLOR_BACKGROUND:
+			gtk_widget_modify_bg (GTK_WIDGET (applet->w_applet),
+					GTK_STATE_NORMAL, color);
+			break;
+		case PANEL_PIXMAP_BACKGROUND:
+			style = gtk_style_copy (
+					GTK_WIDGET (applet->w_applet)->style);
+			if (style->bg_pixmap[GTK_STATE_NORMAL])
+				g_object_unref (
+					style->bg_pixmap[GTK_STATE_NORMAL]);
+			style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (
+					pixmap);
+			gtk_widget_set_style (
+					GTK_WIDGET (applet->w_applet), style);
+			break;
 	}
-	
-	return FALSE;
 }
 
 /* Applet Callback : Deletes the applet. */
@@ -232,7 +253,7 @@ void menu_preferences_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, cons
 void menu_help_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, const gchar *verbname)
 {
 	GError *error = NULL;
-	gnome_help_display_on_screen("stickynotes_applet", "stickynotes-introduction", gtk_widget_get_screen(applet->w_applet), &error);
+	gnome_help_display_on_screen("stickynotes_applet", NULL, gtk_widget_get_screen(applet->w_applet), &error);
 	if (error) {
 		GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 							   _("There was an error displaying help: %s"), error->message);
